﻿using HeuristicLab.Clients.Access;
using HeuristicLab.Clients.Access.Administration;
using HeuristicLab.Clients.Hive.WebJobManager.Services;
using Microsoft.AspNet.Hosting;
using Microsoft.AspNet.Http;
using Microsoft.AspNet.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel.Security;
using System.Threading;
using System.Threading.Tasks;

namespace HeuristicLab.Clients.Hive.WebJobManager.Controllers
{
    public class ResourceController : Controller
    {
        private WebLoginService weblog;
        private HiveServiceLocatorWeb serviceLocator;
        private AccessAdministrationClient accessClient;
        private HiveAdminClientWeb adminClient;
        private Guid userId;

        private IHostingEnvironment _environment;

        public ResourceController(IHostingEnvironment env)
        {
            weblog = WebLoginService.Instance;
            _environment = env;
        }
        private bool init()
        {
            var u = HttpContext.Session.GetString("UserId");
            if (u == null || u == "" || Guid.Parse(u) == Guid.Empty)
            {
                return false;
            }
            else
            {
                userId = Guid.Parse(u);
                serviceLocator = weblog.getServiceLocator(userId);
                adminClient = weblog.getAdminClient(userId);
                accessClient = weblog.getAccessAdminClient(userId);
                return serviceLocator.CheckLogin();

            }
        }
        public IActionResult Index()
        {
            
            if (init())
            {
                ViewBag.SessionId = HttpContext.Session.GetString("UserId");
                ViewBag.Title = "Resources";
                return View("Index");
            }
            else
            {
                return RedirectToAction("Index", "Home");
            }
        }
        [HttpPost]
        public IActionResult changeParent(string inpid, string inpidpar)
        {
            if (init())
            {
                var gid = Guid.Parse(inpid);
                

                adminClient.Refresh();
                var tochange = adminClient.Resources.ToList().Find(x => x.Id == gid);
                
                if (inpidpar != null && inpidpar != "") {
                    Guid tempid = Guid.Parse(inpidpar);
                    while (tempid != null)
                    {
                        if (tempid != gid)
                        {
                            tempid = (Guid)adminClient.Resources.ToList().Find(x => x.Id == tempid).ParentResourceId;
                        }
                        else
                            break;
                    }
                    if(tempid == null)
                        tochange.ParentResourceId = Guid.Parse(inpidpar);
                }
                else
                    tochange.ParentResourceId = null;
                adminClient.Store(tochange, CancellationToken.None);

                return RedirectToAction("Index", "Resource");
            }
            else
            {
                return RedirectToAction("Index", "Home");
            }
        }
        [HttpPost]
        public IActionResult deleteClientGroup(string inpid)
        {
            if (init())
            {
                var gid = Guid.Parse(inpid);

                adminClient.Refresh();
                var group = adminClient.Resources.ToList().Find(x => x.Id == gid);
                var list = adminClient.Resources.ToList().FindAll(x => x.ParentResourceId == gid);
                foreach(var cl in list)
                {
                    cl.ParentResourceId = group.ParentResourceId;
                    adminClient.Store(cl, CancellationToken.None);
                }
                adminClient.Delete(group);

                return RedirectToAction("Index", "Resource");
            }
            else
            {
                return RedirectToAction("Index", "Home");
            }
        }

        [HttpPost]
        public IActionResult newClientGroup(string inpname, int inpheart, string inpparent, string[] clientstoadd)
        {
            if (init()) {
                adminClient.Refresh();

                SlaveGroup sg = new SlaveGroup();
                sg.Id = Guid.Empty;
                sg.Name = inpname;
                sg.HbInterval = inpheart;
                if (inpparent != "")
                    sg.ParentResourceId = Guid.Parse(inpparent);
                sg.OwnerUserId = serviceLocator.getHiveServiceClient().GetUserIdByUsername(serviceLocator.getHiveServiceClient().ClientCredentials.UserName.UserName);
                adminClient.Store(sg, CancellationToken.None);
                adminClient.Refresh();
                var id = adminClient.Resources.ToList().Find(x => x.Name == inpname).Id;
                foreach(var s in clientstoadd)
                {
                    var cid = Guid.Parse(s);
                    var client = adminClient.Resources.ToList().Find(x => x.Id == cid);
                    client.ParentResourceId = id;
                    adminClient.Store(client, CancellationToken.None);
                }

                return RedirectToAction("Index", "Resource");
            }
            else
            {
                return RedirectToAction("Index", "Home");
            }
        }
    }
}
